//
// color defintions for player 3
//
#declare helmetColor    = color rgb <0,0,0>;
#declare faceColor      = color rgb <0,0,0>;
#declare bodyColor      = color rgb <0,0,0>;
#declare handsFeetColor = color rgb <0,0,0>;
#declare armsLegsColor  = color rgb <1,0,0>;
#declare backpackColor  = color rgb <0,1,0>;

#declare floorColor = color rgb <0.333,0.333,0.333>;
