
#declare playerStanding = 0;
#declare playerWalking = 0;
#declare playerWinning = 0;
#declare playerLosing = 0;
#declare playerKilled = 0;
#declare playerAnimationClock = 0;

#declare playerLocation = 0;
#declare playerAngle = 0;

